<?php

/**
 * @file
 * Roster management functions related to the ULT_Rosters component of the Ultimate League Tool.
 */

/********************************************************
 *                                                      *
 *                 Page Declarations                    *
 *                                                      *
 ********************************************************/
/**
 * Generates the per team 'Invites/Apps' page
 *
 * @param $rosternode
 *   A node object representing the current team's roster
 *
 * @return $output
 *   The rendered Invites/Apps page
 */
function ult_rosters_inviteapp_page($rosternode) {
  _ult_main_checktype($rosternode, 'ult_roster');
  $output .= '<fieldset class="collapsible"><legend>' . t('Roster Status') . '</legend>';
  $output .= '<div>';
  if ($rosternode->field_ult_rosterstatus[0]['value'] == 'Open') {
    $output .= t('This roster is currently open to player applications.  ');
    $output .= t('If you wish to change this setting, please use the button below.  ');
    $output .= '<br /><br />';
    $output .= drupal_get_form('ult_rosters_closeapps_form', $rosternode);
  }
  elseif ($rosternode->field_ult_rosterstatus[0]['value'] == 'Closed') {
    $output .= t('This roster is currently closed to player applications.  ');
    $output .= t('If you wish to change this setting, please use the button below.  ');
    $output .= '<br /><br />';
    $output .= drupal_get_form('ult_rosters_openapps_form', $rosternode);
  }
  elseif ($rosternode->field_ult_rosterstatus[0]['value'] == 'Locked') {
    $output .= t('This roster is currently locked.  ');
    $output .= t('No further changes (including roster invites/applications) are allowed.');
  }
  $output .= '</div></fieldset>';
  if ($rosternode->field_ult_rosterstatus[0]['value'] == 'Locked') {
    return $output;
  }
  // Determine whether invite forms allowed for this user
  global $user;
  if ((user_access('administer ULT leagues'))
       || ((variable_get('ult_enablecaptains', 1) == 1) && (variable_get('ult_captainrosteradd', 'invite') != 'none') && (_ult_main_iscaptain($rosternode, $user->uid)))) {
    // Invite player by name
    $output .= drupal_get_form('ult_rosters_invitebyname_form', $rosternode);
    // Invite player by email
    $output .= drupal_get_form('ult_rosters_invitebyemail_form', $rosternode);
    // Retrive invites
    $invites = _ult_rosters_getinvitesbyroster($rosternode);
    // Current Invites (if required)
    if (($invites['invites'] != array()) || ($invites['email'] != array())) {
      $output .= '<fieldset class="collapsible"><legend>' . t('Pending Roster Invitations') . '</legend>';
      if (count($invites['invites'] > 0) && ($invites['invites'] != array())) {
        $header = array(t("Player"), t("Invite By"), t("Invite Date"), t("Response"), t("Actions"));
        $output .= theme_table($header, $invites['invites']);
      }
      if (count($invites['email'] > 0) && ($invites['email'] != array())) {
        $header = array(t("Player"), t("Email"), t("Invite By"), t("Invite Date"), t("Actions") );
        $output .= theme_table($header, $invites['email']);
      }
      $output .= '</fieldset><br />';
    }
    // Applications (if required)
    if (count($invites['apps']) > 0) {
      $output .= '<fieldset class="collapsible"><legend>Pending Roster Applications</legend><div>';
      $header = array(t("Player"), t("Application Date"), t("Response"), t("Actions"));
      $output .= theme_table($header, $invites['apps']);
      $output .= '</div></fieldset><br />';
    }
  }
  return $output;
}

/********************************************************
 *                                                      *
 *                 Form Declarations                    *
 *                                                      *
 ********************************************************/

/**
 * Implements hook_form().
 *
 * @param $form_state
 *   The form state array.
 * @param $rosternode
 *   A Roster node object as passed by _ult_rosters_rosterbyteamnid_load()
 *
 * @return
 *   An array containing the form elements to be displayed in the node edit form.
 */
function ult_rosters_invitebyname_form($form_state, $rosternode) {
  _ult_main_checktype($rosternode, 'ult_roster');
  $rosternid = $rosternode->nid;
  $teamnid = $rosternode->field_ult_rosterteam[0]['nid'];
  $form = array();
  $description = t('Use this section to invite players who already have an account and player profile on this website.');
  $form['invitecontainer'] = array( '#title' => t('Invite Players (by name)'), '#type' => 'fieldset', '#description' => $description, '#collapsible' => TRUE, '#collapsed' => FALSE, );
  $description = t('Enter the name of the player (as entered in their player profile) that you wish to invite to your roster.');
  $form['invitecontainer']['ultrosteruser'] = array( '#type' => 'textfield', '#title' => t('Name'), '#description' => $description, '#autocomplete_path' => 'ult/playername/autocomplete', '#size' => 30, );
  $form['rosterid'] = array( '#type' => 'hidden', '#value' => $rosternid );
  $form['teamid'] = array( '#type' => 'hidden', '#value' => $teamnid, );
  $form['invitecontainer']['submit'] = array(  '#type' => 'submit', '#value' => t('Invite Player'), );
  return $form;
}

/**
 * Implements hook_form().
 *
 * @param $form_state
 *   The form state array.
 * @param $rosternode
 *   A Roster node object as passed by _ult_rosters_rosterbyteamnid_load()
 *
 * @return
 *   An array containing the form elements to be displayed in the node edit form.
 */
function ult_rosters_invitebyemail_form($form_state, $rosternode) {
  _ult_main_checktype($rosternode, 'ult_roster');
  $rosternid = $rosternode->nid;
  $teamnid = $rosternode->field_ult_rosterteam[0]['nid'];
  $form = array();
  $description = t('Use this section to invite players who have not already created an account on this website.');
  $form['invitecontainer'] = array( '#title' => 'Invite Players (by email)', '#type' => 'fieldset', '#description' => $description, '#collapsible' => TRUE, '#collapsed' => FALSE, );
  $description = t('Enter the name and an email address for the player you wish to invite to your roster.');
  $form['invitecontainer']['usage'] = array(  '#type' => 'markup', '#value' => '', '#description' => $description, );
  $form['invitecontainer']['ult_firstname'] = array( '#type' => 'textfield', '#title' => t('First Name'), '#size' => 30, );
  $form['invitecontainer']['ult_lastname'] = array( '#type' => 'textfield', '#title' => t('Last Name'), '#size' => 30, );
  $form['invitecontainer']['ult_email'] = array( '#type' => 'textfield', '#title' => t('Email Address'), '#autocomplete_path' => 'ult/playeremail/autocomplete', '#size' => 30, );
  $form['rosterid'] = array( '#type' => 'hidden', '#value' => $rosternid, );
  $form['teamid'] = array( '#type' => 'hidden', '#value' => $teamnid, );
  $form['confirmeduid'] = array('#type' => 'value', '#value' => NULL, );
  $form['invitecontainer']['submit'] = array(  '#type' => 'submit', '#value' => t('Invite Player'), );
  return $form;
}

/**
 * Implements hook_form().
 *
 * @param $form_state
 *   The form state array.
 * @param $rosternode
 *   A Roster node object as passed by _ult_rosters_rosterbyteamnid_load()
 *
 * @return
 *   An array containing the form elements to be displayed in the node edit form.
 */
function ult_rosters_openapps_form($form_state, $rosternode) {
  _ult_main_checktype($rosternode, 'ult_roster');
  $form = array();
  $form['rosterid'] = array( '#type' => 'hidden', '#value' => $rosternode->nid, );
  $form['submit'] = array(  '#type' => 'submit', '#value' => t('Enable Player Applications'), );
  return $form;
}

/**
 * Implements hook_form().
 *
 * @param $form_state
 *   The form state array.
 * @param $rosternode
 *   A Roster node object as passed by _ult_rosters_rosterbyteamnid_load()
 *
 * @return
 *   An array containing the form elements to be displayed in the node edit form.
 */
function ult_rosters_closeapps_form($form_state, $rosternode) {
  _ult_main_checktype($rosternode, 'ult_roster');
  $form = array();
  $form['rosterid'] = array( '#type' => 'hidden', '#value' => $rosternode->nid, );
  $form['submit'] = array(  '#type' => 'submit', '#value' => t('Disable Player Applications'), );
  return $form;
}

/********************************************************
 *                                                      *
 *                 Form Validation                      *
 *                                                      *
 ********************************************************/

/**
 * Validates the ult_rosters_invitebyname_form form
 *
 * @param $form
 *   form to validate
 * @param $form_state
 *   A keyed array containing the current state of the form.
 */
function ult_rosters_invitebyname_form_validate($form, &$form_state) {

}

/**
 * Validates the ult_rosters_invitebyemail_form form
 *
 * @param $form
 *   form to validate
 * @param $form_state
 *   A keyed array containing the current state of the form.
 */
function ult_rosters_invitebyemail_form_validate($form, &$form_state) {
  $firstname = trim(filter_xss($form_state['values']['ult_firstname']));
  $lastname = trim(filter_xss($form_state['values']['ult_lastname']));
  $email = check_plain($form_state['values']['ult_email']);
  if (($email == '') || (!valid_email_address($email))) {
    form_set_error('ult_email', t("Please enter a valid email address for the player."));
    return;
  }
  if ($firstname == '') {
    form_set_error('ult_firstname', t("Please enter the player's first name."));
    return;
  }
  if ($lastname == '') {
    form_set_error('ult_lastname', t("Please enter the player's last name."));
    return;
  }
  $playername = $firstname . ' ' . $lastname;
  $uid = 0;
  if (variable_get('ult_rostersinvitebyname', 'name') == 'name') {
    // Check for account
    $validusers = _ult_rosters_player_autocomplete($playername, "validate");
    $userindex = $validusers['ultplayerindex'];
    unset($validusers['ultplayerindex']);
    if ((count($validusers) == 1) && ($playername == $validusers[$playername])) {
      // Account match found based on name
      $uid = $userindex[$playername];
      $target = user_load($uid);
      if ($target->mail != $email) {
        // Email address doesn't match
        form_set_error('form', t("Our records indicate that !playername is using a different email address (!mail) for this site.  Please update the form and try again.", array('!playername' => $playername, '!mail' => $target->mail)));
        return;
      }
      form_set_value($form['confirmeduid'], $uid, $form_state);
    }
  }
  if ($uid == 0) {
    // No account matched based on name.  Check email
    $validemails = _ult_rosters_email_autocomplete($email, "validate");
    $emailindex = $validemails['ultemailindex'];
    unset($validemails['ultemailindex']);
    if ((count($validemails == 1)) && ($email == $validemails[$email])) {
      // Account match found based on email
      $uid = $emailindex[$email];
      $target = user_load($uid);
      if (variable_get('ult_rostersinvitebyname', 'name') == 'name') {
        // Player may not have a profile filled out yet.  If they do, check that name matches
        if ((($target->field_ult_firstname != '') && ($target->field_ult_firstname != $firstname))
        || (($target->field_ult_lastname != '') && ($target->field_ult_lastname != $lastname))) {
          // Has profile, but name mismatch
          form_set_error('form', t("Our records indicate that email address belongs to !player. Please correct the name or enter a different email address for your invite.", array('!player' => theme('username', $target))));
          return;
        }
      }
      // No profile (if name matches, it would have been caught above.)  Assume correct user.
      form_set_value($form['confirmeduid'], $uid, $form_state);
    }
  }
}


/********************************************************
 *                                                      *
 *                 Form Submission                      *
 *                                                      *
 ********************************************************/
/**
 * Processes the submitted ult_rosters_invitebyname_form form
 *
 * @param $form
 *   form to submit
 * @param $form_state
 *   A keyed array containing the current state of the form.
 */
function ult_rosters_invitebyname_form_submit($form, &$form_state) {
  $username = check_plain($form_state['values']['ultrosteruser']);
  $validusers = _ult_rosters_player_autocomplete($username, "validate");
  $userindex = $validusers['ultplayerindex'];
  unset($validusers['ultplayerindex']);
  if ((count($validusers) == 1) && ($username == $validusers[$username])) {
    $uid = $userindex[$username];
    $nid = check_plain($form_state['values']['rosterid']);
    if (_ult_rosters_ismember($nid, $uid)) {
      drupal_set_message(t("That player is already a member of this roster."));
      return;
    }
    global $user;
    if (user_access('administer ULT leagues') && variable_get('ult_adminrosteradd', 'direct') == 'direct') {
      _ult_rosters_acceptinvite($uid, node_load($nid));
    }
    elseif ((variable_get('ult_enablecaptains', 1) == 1) && (variable_get('ult_captainrosteradd', 'invite') == 'direct')) {
      $rosternode = node_load($nid);
      $teamnode = node_load($rosternode->field_ult_rosterteam[0]['nid']);
      if (_ult_main_iscaptain($teamnode, $user->uid)) {
        _ult_rosters_acceptinvite($uid, $rosternode);
      }
      else {
        _ult_rosters_createinvite($uid, $nid, $user->uid);
      }
    }
    else {
      _ult_rosters_createinvite($uid, $nid, $user->uid);
    }
    return;
  }
  drupal_set_message(t("Unable to uniquely identify an account with that player name.  Please provide a valid email address which we may use to pass on your invitation."));
}

/**
 * Processes the submitted ult_rosters_invitebyemail_form form
 *
 * @param $form
 *   form to submit
 * @param $form_state
 *   A keyed array containing the current state of the form.
 */
function ult_rosters_invitebyemail_form_submit($form, &$form_state) {
  $firstname = trim(filter_xss($form_state['values']['ult_firstname']));
  $lastname = trim(filter_xss($form_state['values']['ult_lastname']));
  $email = check_plain($form_state['values']['ult_email']);
  $rosternid = check_plain($form_state['values']['rosterid']);
  if ($form_state['values']['confirmeduid'] != '') {
    // Player ID Verified in validation routine.
    $uid = $form_state['values']['confirmeduid'];
    $target = user_load($uid);
    global $user;
    if ($existing = _ult_rosters_hasinvite($uid, $rosternid)) {
      drupal_set_message(t('That player already has an existing invitation to join this team.'));
    }
    elseif (_ult_rosters_ismember($rosternid, $uid)) {
      drupal_set_message(t('That player is already a member of this team\'s roster.'));
    }
    else {
      _ult_rosters_createinvite($uid, $rosternid, $user->uid);
      drupal_set_message(t('Roster invitation sent to ') . ult_playername_theme($uid, TRUE) . '.');
    }
  }
  else {
    drupal_set_message(t("That player does not currently have an account on this website.  They will be emailed an invitation to create an account and join your roster."));
    // Send 'unknown' invitation
    if (!(_ult_rosters_hasunknowninvite($email, $rosternid))) {
      global $user;
      $sql = "Insert into {ult_roster_unknown_invites} (roster, referrer, invitedate, email, firstname, lastname) values (%d, %d, %d, '%s', '%s', '%s')";
      $result = db_query($sql, $rosternid, $user->uid, time(), trim($email), trim($firstname), trim($lastname));
      $params = array();
      $params['teamname'] = check_plain(node_page_title(node_load($rosternid)));
      $params['userid'] = $user->uid;
      drupal_mail('ult_rosters', 'email_notify_player', $email, language_default(), $params);
    }
    else {
      drupal_set_message(t('That player already has an existing invitation to join this team.'));
    }
  }
}

/**
 * Processes the submitted ult_rosters_openapps_form form
 *
 * @param $form
 *   form to submit
 * @param $form_state
 *   A keyed array containing the current state of the form.
 */
function ult_rosters_openapps_form_submit($form, &$form_state) {
  $nid = check_plain($form_state['values']['rosterid']);
  $rosternode = node_load($nid);
  _ult_main_checktype($rosternode, 'ult_roster');
  $rosternode->field_ult_rosterstatus[0]['value'] = 'Open';
  $_SESSION['ult_rosters_skipnodeapi'] = TRUE;
  node_save($rosternode);
  unset($_SESSION['ult_rosters_skipnodeapi']);
}


/**
 * Processes the submitted ult_rosters_closeapps_form form
 *
 * @param $form
 *   form to submit
 * @param $form_state
 *   A keyed array containing the current state of the form.
 */
function ult_rosters_closeapps_form_submit($form, &$form_state) {
  $nid = check_plain($form_state['values']['rosterid']);
  $rosternode = node_load($nid);
  _ult_main_checktype($rosternode, 'ult_roster');
  $rosternode->field_ult_rosterstatus[0]['value'] = 'Closed';
  $_SESSION['ult_rosters_skipnodeapi'] = TRUE;
  node_save($rosternode);
  unset($_SESSION['ult_rosters_skipnodeapi']);
}

/********************************************************
 *                                                      *
 *                 Private Functions                    *
 *                                                      *
 ********************************************************/
/**
 * Generates the invitation table rows for a given roster
 *
 * @param $roster
 *   A node object representing the given roster
 *
 * @return $output
 *   The rendered tables rows ready for passing into theme_table()
 */
function _ult_rosters_getinvitesbyroster($roster) {
  // Row variable for Invitations
  $inviterow = array();
  // Row variable for Applications
  $appsrow = array();
  // Row variable for Email invites
  $emailrow = array();
  if ($invites = _ult_rosters_getinvites( NULL, $roster->nid)) {
    $i=0;
    $j=0;
    foreach ($invites as $invite) {
      if ($invite['uid'] != $invite['referrerid']) {
        // Invitation
        $inviterow[$i]['uid'] = ult_playername_theme($invite['uid'], TRUE);
        $inviterow[$i]['referrerid'] = ult_playername_theme($invite['referrerid'], TRUE);
        $inviterow[$i]['invitedate'] = format_date($invite['invitedate'], "small");
        if ($invite['response'] == 1) {
          // Has been declined
          $inviterow[$i]['response'] = t("Invitation declined on ") . format_date($invite['rejectdate'], "small");
          $inviterow[$i]['actions'] = l(t('Delete'), 'ult/team/' . $roster->field_ult_rosterteam[0]['nid'] . '/roster/invites/removeinvite/' . $invite['uid']);
        }
        else {
          // Awaiting Response
          $inviterow[$i]['response'] = t("Awaiting Response");
          $inviterow[$i]['actions'] = l(t('Withdraw Invitation'), 'ult/team/' . $roster->field_ult_rosterteam[0]['nid'] . '/roster/invites/withdrawinvite/' . $invite['uid']);
        }
        $i++;
      }
      else {
        // Application
        $appsrow[$j]['uid'] = ult_playername_theme($invite['uid'], TRUE);
        $appsrow[$j]['invitedate'] = format_date($invite['invitedate'], "small");
        if ($invite['response'] == 1) {
          // Has been declined
          $appsrow[$j]['response'] = t("Application declined on ") . format_date($invite['rejectdate'], "small");
          // Determine roster app handling method
          if (variable_get('ult_rosterapphandling', 'invite') == 'invite') {
            $appsrow[$j]['actions'] = l(t("Invite"), 'ult/team/' . $roster->field_ult_rosterteam[0]['nid'] . '/roster/invites/acceptapp/' . $invite['uid']);
          }
          else {
            $appsrow[$j]['actions'] = l(t("Accept"), 'ult/team/' . $roster->field_ult_rosterteam[0]['nid'] . '/roster/invites/acceptapp/' . $invite['uid']);
          }
          if (time() - (60*60*24*7) > $invite['rejectdate']) {
            // Player has had a week to see the application rejection, provide link to allow team captain to delete the application
            $appsrow[$j]['actions'] .= "&nbsp;&nbsp;&nbsp;";
            $appsrow[$j]['actions'] .= l(t("Delete"), 'ult/team/' . $roster->field_ult_rosterteam[0]['nid'] . '/roster/invites/removeapp/' . $invite['uid']);
          }
        }
        else {
          // Player still awaiting response.  Provide Accept/Delete links
          $appsrow[$j]['response'] = t("Player Awaiting Response");
          // Determine roster app handling method
          if (variable_get('ult_rosterapphandling', 'invite') == 'invite') {
            $appsrow[$j]['actions'] = l(t("Invite"), 'ult/team/' . $roster->field_ult_rosterteam[0]['nid'] . '/roster/invites/acceptapp/' . $invite['uid']);
          }
          else {
            $appsrow[$j]['actions'] = l(t("Accept"), 'ult/team/' . $roster->field_ult_rosterteam[0]['nid'] . '/roster/invites/acceptapp/' . $invite['uid']);
          }
          $appsrow[$j]['actions'] .= "&nbsp;&nbsp;&nbsp;";
          $appsrow[$j]['actions'] .= l(t("Decline"), 'ult/team/' . $roster->field_ult_rosterteam[0]['nid'] . '/roster/invites/rejectapp/' . $invite['uid']);
        }
        $j++;
      }
    }
  }
  // Check 'unknown' invites table
  $i = 0;
  if ($invites = _ult_rosters_getemailinvites($roster->nid)) {
    foreach ($invites as $invite) {
      $emailrow[$i] = array( $invite['firstname'] . " " . $invite['lastname'], $invite['email'],  theme('username', user_load($invite['referrer'])), format_date($invite['invitedate'], "small"));
      $emailrow[$i]['actions'] = l(t("Withdraw Invitation"), 'ult/team/' . $roster->field_ult_rosterteam[0]['nid'] . '/roster/invites/withdrawemailinvite/' . urlencode($invite['firstname']) . urlencode($invite['lastname']));
      $i++;
    }
  }
  return array('invites' => $inviterow, 'apps' => $appsrow, 'email' => $emailrow);
}

/**
 * Creates a particular roster invitation
 *
 * @param $uid
 *   The uid of the user invited to join a particular roster
 * @param $rosterid
 *   The nid of the roster the invite is for
 *
 * @return
 */
function _ult_rosters_createinvite($uid, $rosterid, $referrer) {
  $uid = check_plain($uid);
  $rosterid = check_plain($rosterid);
  $referrer = check_plain($referrer);
  if (_ult_rosters_hasinvite($uid, $rosterid)) {
      drupal_set_message(t('That player has already been invited to join this roster.'));
      return;
  }
  $sql = "INSERT into {ult_roster_invites} (uid, roster, referrer, invitedate) values (%d, %d, %d, %d)";
  $result = db_query($sql, $uid, $rosterid, $referrer, time());
  $referuser = user_load($referrer);
  $params = array();
  $params['teamname'] = check_plain(node_page_title(node_load($rosterid)));
  $params['user_realname'] = theme('username', $referuser,  array("plain" => TRUE));
  $params['userid'] = $uid;
  $useracct = user_load($uid);
  // Send invitation email
  drupal_mail('ult_rosters', 'invite_notify_player', $useracct->mail, user_preferred_language($useracct), $params, $referuser->mail);
  drupal_set_message(t('Roster invitation created.'));
}

/**
 * Generates the User/Uid/Invites page
 *
 * @param $account
 *   The result of user_load($int) where $int is UID argument from the menu item. Should result in a $user object
 *
 * @return page content
 */
function ult_rosters_playerinvites_page($account) {
  if (module_exists('ult_waivers')) {
    ult_waivers_check_waiver();
  }
  // Retrieve Invite Details
  if ($invites = _ult_rosters_getinvites($account->uid)) {
    foreach ($invites as $invite) {
      if ($invite['uid'] != $invite['referrerid']) {
        if ($invite['rejectdate'] != '') {
          $actionlinks = l(t('Accept'), 'user/' . $account->uid . '/invites/acceptinvite/' . $invite['rosterid']) . "&nbsp;&nbsp;&nbsp;";
          if (time() - (60*60*24*7) > $invite['rejectdate']) {
            $actionlinks .= l(t('Delete'), 'user/' . $account->uid . '/invites/deleteinvite/' . $invite['rosterid']);
          }
          $rows[] = array($invite['teamname'], $invite['referrer'], format_date($invite['invitedate'], "small"), $actionlinks, t('Declined ') . format_date($invite['rejectdate'], "small"));
        }
        else {
        $actionlinks = l(t('Accept'), 'user/' . $account->uid . '/invites/acceptinvite/' . $invite['rosterid']) . "&nbsp;&nbsp;&nbsp;" . l(t('Decline'), 'user/' . $account->uid . '/invites/rejectinvite/' . $invite['rosterid']);
          $rows[] = array($invite['teamname'], $invite['referrer'], format_date($invite['invitedate'], "small"), $actionlinks, '');
        }
      }
      else {
        if ($invite['rejectdate'] != '') {
          $actionlinks = l(t('Delete'), 'user/' . $account->uid . '/invites/removeapp/' . $invite['rosterid']);
          $rows2[] = array($invite['teamname'], format_date($invite['invitedate'], "small"), $actionlinks, t('Declined ') . format_date($invite['rejectdate'], "small"));
        }
        else {
          $actionlinks = l(t('Withdraw request'), 'user/' . $account->uid . '/invites/withdrawapp/' . $invite['rosterid']);
          $rows2[] = array($invite['teamname'], format_date($invite['invitedate'], "small"), $actionlinks, t('Awaiting Response'));
        }
      }
    }
    if (count($rows) > 0) {
      $headers = array(t('Team'), t('Sent by'), t('Date'), t('Action'), '');
      $output .= '<div class="ult_table_title"><h3>' . t('Your Invitations') . '</h3></div>';
      $output .= theme_table($headers, $rows);
      $output .= '<br /><br />';
    }
    if (count($rows2) > 0) {
      $headers = array(t('Team'), t('Date'), t('Action'), t('Result'));
      $output .= '<div class="ult_table_title"><h3>' . t('Your Applications') . '</h3></div>';
      $output .= theme_table($headers, $rows2);
      $output .= '<br /><br />';
    }
  }
  else {
    $output .= t("There are currently no outstanding roster invitations/applications for this account.");
  }
  return $output;
}

/**
 * Callback handler for User/Uid/Invites/Op/Roster page
 *
 * @param $usernode
 *   The result of user_load($int) where $int is UID argument from the menu item. Should result in a $user object
 * @param $op
 *   The desired invite operation (accept, decline, remove, withdraw, or delete) + (application or invitation)
 * @param $rosternode
 *   The result of rufdcroster_roster_nid_load($int) where $int is the roster argument from the menu item.  Should result in a rufdc_roster object.
 *
 * @return $ouput
 *   Rendered page content
 */
function _ult_rosters_playerinvitehandler($usernode, $op, $rosternode) {
  _ult_main_checktype($rosternode, 'ult_roster');
  // Retrive Invite Details
  if ($invite = _ult_rosters_getinvites($usernode->uid, $rosternode->nid)) {
    switch ($op) {
      case 'acceptinvite':
        _ult_rosters_acceptinvite($usernode->uid, $rosternode);
        break;
      case 'rejectinvite':
        _ult_rosters_rejectinvite($usernode->uid, $rosternode);
        break;
      case 'removeinvite':
        _ult_rosters_deleteinvite($usernode->uid, $rosternode);
        break;
      case 'withdrawapp':
        _ult_rosters_withdrawapp($usernode->uid, $rosternode);
        break;
      case 'removeapp':
        _ult_rosters_deleteapp($usernode->uid, $rosternode);
        break;
      default;
    }
  }
  drupal_goto('user/' . $usernode->uid . '/invites');
}

/**
 * Callback handler for ult/team/TEAMNID/Roster/invites/Op/Uid page
 *
 * @param $usernode
 *   The result of _ult_rosters_rosterbyteamnid_load($int) where $int is the team nid argument from the menu item.  Should result in a 'ULT_ROSTER' node object.
 * @param $op
 *   The desired invite operation (accept, decline, remove, withdraw, or delete) + (application or invitation)
 * @param $invite
 *   Details regarding which invitation to perform the given action on. May be a user ID or text name if an email invite.
 *
 * @return $output
 *   Rendered page content
 */
function _ult_rosters_teaminvitehandler($rosternode, $op, $invitedetails) {
  _ult_main_checktype($rosternode, 'ult_roster');
  // Retrive Invite Details
  if (!($invitedetails))  {
    drupal_set_message(t('The site encountered a problem while attempting that action.  Please email the site administrator with details regarding the action you were attempting, as well as the team and player name involved in the attempt.'));
    drupal_goto('ult/team/' . $rosternode->field_ult_rosterteam[0]['nid'] . '/roster/invites');
    return;
  }
  // Verify we're passed a valid userid (which has a corresponding invite)
  // Except in the case of a email invite
  if ($op != 'withdrawemailinvite') {
    $usernode = user_load(check_plain($invitedetails));
    if (!($invite = _ult_rosters_getinvites($usernode->uid, $rosternode->nid))) {
      drupal_goto('ult/team/' . $rosternode->field_ult_rosterteam[0]['nid'] . '/roster/invites');
      return;
    }
  }
  switch ($op) {
    case 'withdrawinvite':
      _ult_rosters_withdrawinvite($usernode->uid, $rosternode);
      break;
    case 'removeinvite':
      _ult_rosters_deleteinvite($usernode->uid, $rosternode);
      break;
    case 'acceptapp':
      _ult_rosters_acceptapp($usernode->uid, $rosternode);
      break;
    case 'rejectapp':
      _ult_rosters_rejectapp($usernode->uid, $rosternode);
      break;
    case 'removeapp':
      _ult_rosters_deleteapp($usernode->uid, $rosternode);
      break;
    case 'withdrawemailinvite':
      _ult_rosters_withdrawemailinvite($rosternode->nid, check_plain($invitedetails));
    default:
  }
  drupal_goto('ult/team/' . $rosternode->field_ult_rosterteam[0]['nid'] . '/roster/invites');
}

/**
 * Adds a player to a particular roster
 *
 * @param $uid
 *   The uid of the user for which you want to retrive the roster join requests
 * @param $rosterid
 *   The nid of the roster for which you want to add that player to
 *
 */
function _ult_rosters_acceptinvite($uid, $rosternode) {
  global $user;
  // Add Player to Roster
  // Check that player is not already on roster
  if (in_array(array('uid' => $uid), $rosternode->field_ult_rosterplayers)) {
      drupal_set_message(t('!player is already a member of the @roster.', array('!player' => theme('username', user_load($uid)), '@roster' => $rosternode->title)));
  }
  else {
    // Check waiver if necessary
    if ((module_exists('ult_waivers')) && (variable_get('ult_enablewaiver', 0) > 0)) {
      ult_waivers_check_waiver();
    }
    // Process invite acceptance
    $rosternode->field_ult_rosterplayers[] = array('uid' => $uid);
    drupal_set_message(t('!player has been added to the @roster.', array('!player' => ult_playername_theme($uid, TRUE), '@roster' => $rosternode->title)));
    $_SESSION['ult_rosters_skipnodeapi'] = TRUE;
    node_save($rosternode);
    unset($_SESSION['ult_rosters_skipnodeapi']);
    // Determine whether 'new player joined' notifications are required
    $targets = array();
    if (variable_get('ult_enablecaptains', 1) == 1) {
      $targets = variable_get('ult_joinnotify', array('player' => 'player', 'captain' => 'captain'));
    }
    else {
      $targets = variable_get('ult_joinnotify', array('player' => 'player'));
    }
    if (($targets['player'] === 'player') || ($targets['captain'] === 'captain') || ($targets['team'] === 'team')) {
      drupal_set_message(t('Email notifications sent.'), 'status');
      $params = array();
      $params['teamnid'] = $rosternode->field_ult_rosterteam[0]['nid'];
      $params['teamname'] = check_plain(node_page_title(node_load($params['teamnid'])));
      $params['name'] = ult_playername_theme($uid, FALSE);
      $params['userid'] = $uid;
      $teamnode = node_load($rosternode->field_ult_rosterteam[0]['nid']);
      _ult_rosters_sendnotifications($targets, $params, 'accept', $rosternode, $teamnode);
    }
  }
  // Remove from 'pending' roster (ie. delete invites or apps)
  _ult_rosters_deleteapp($uid, $rosternode);
  _ult_rosters_deleteinvite($uid, $rosternode);
}

/**
 * Allows a player to reject a particular roster invitation
 *
 * @param $uid
 *   The uid of the user invited to join a particular roster
 * @param $rosterid
 *   The nid of the roster the invite is for
 */
function _ult_rosters_rejectinvite($uid, $rosternode) {
  global $user;
  $sql = "UPDATE {ult_roster_invites} set response=1, rejectdate=" . time() . " WHERE uid=%d AND roster=%d AND referrer!=%d";
  $result = db_query($sql, $uid, $rosternode->nid, $uid);
  drupal_set_message(t("Roster invitation for !player declined.", array('!player' => ult_playername_theme($uid, TRUE))));
  // Determine whether 'new player joined' notifications are required
  $targets = array();
  if (variable_get('ult_enablecaptains', 1) == 1) {
    $targets = variable_get('ult_rejectinvitenotify', array('captain' => 'captain'));
  }
  else {
    $targets = variable_get('ult_rejectinvitenotify', array());
  }
  if (($targets['player'] === 'player') || ($targets['captain'] === 'captain') || ($targets['team'] === 'team')) {
    drupal_set_message(t('Email notifications sent.'), 'status');
    $params = array();
    $params['teamnid'] = $rosternode->field_ult_rosterteam[0]['nid'];
    $params['teamname'] = check_plain(node_page_title(node_load($params['teamnid'])));
    $params['name'] = ult_playername_theme($uid, FALSE);
    $params['userid'] = $uid;
    $teamnode = node_load($rosternode->field_ult_rosterteam[0]['nid']);
    _ult_rosters_sendnotifications($targets, $params, 'rejectinvite', $rosternode, $teamnode);
  }
}

/**
 * Allows a team to withdraw a particular roster invitation
 *
 * @param $uid
 *   The uid of the user invited to join a particular roster
 * @param $rosterid
 *   The nid of the roster the invite is for
 */
function _ult_rosters_withdrawinvite($uid, $rosternode) {
  _ult_main_checktype($rosternode, 'ult_roster');
  global $user;
  $sql = "DELETE from {ult_roster_invites} WHERE uid=%d AND roster=%d and referrer <> %d";
  $result = db_query($sql, $uid, $rosternode->nid, $uid);
  drupal_set_message(t("Roster application for !player withdrawn.", array('!player' => ult_playername_theme($uid, TRUE))));
  // Determine whether 'invitation withdrawn' notifications are required
  $targets = variable_get('ult_withdrawinvitenotify', array('player' => 'player'));
  if (($targets['player'] === 'player') || ($targets['captain'] === 'captain') || ($targets['team'] === 'team')) {
    drupal_set_message(t('Email notifications sent.'), 'status');
    $params = array();
    $params['teamnid'] = $rosternode->field_ult_rosterteam[0]['nid'];
    $teamnode = node_load($params['teamnid']);
    $params['teamname'] = check_plain($teamnode->title);
    $params['name'] = ult_playername_theme($uid, FALSE);
    $params['userid'] = $uid;
    _ult_rosters_sendnotifications($targets, $params, 'withdrawinvite', $rosternode, $teamnode);
  }
}

/**
 * Deletes a particular roster invitation
 *
 * @param $uid
 *   The uid of the user invited to join a particular roster
 * @param $rosterid
 *   The nid of the roster the invite is for
 *
 */
function _ult_rosters_deleteinvite($uid, $rosternode) {
  $sql = "DELETE from {ult_roster_invites} WHERE uid=%d AND roster=%d and referrer <> %d";
  $result = db_query($sql, $uid, $rosternode->nid, $uid);
}

/**
 * Allows a team to accept a player's application (and create a corresponding invite)
 *
 * @param $uid
 *   The uid of the user invited to join a particular roster
 * @param $rosterid
 *   The nid of the roster the invite is for
 */
function _ult_rosters_acceptapp($uid, $rosternode) {
  global $user;
  // Remove any existing records.  Player applications are deleted, and any outstanding or declined invitations are superceded by the current one.
  _ult_rosters_deleteapp($uid, $rosternode);
  _ult_rosters_deleteinvite($uid, $rosternode);
  global $user;
  $sql = "INSERT INTO {ult_roster_invites} (uid, roster, referrer, invitedate, response, rejectdate) values (%d, %d, %d," . time() . ", NULL, NULL)";
  $result = db_query($sql, $uid, $rosternode->nid, $user->uid);
  // Check the roster application handling method setting
  if (variable_get('ult_rosterapphandling', 'invite') == 'direct') {
    _ult_rosters_acceptinvite($uid, $rosternode);
  }
  else {
    drupal_set_message(t("Roster invitation created for !player", array('!player' => ult_playername_theme($uid, TRUE))));
  }
}

/**
 * Allows a team to reject a player's application
 *
 * @param $uid
 *   The uid of the user invited to join a particular roster
 * @param $rosterid
 *   The nid of the roster the invite is for
 */
function _ult_rosters_rejectapp($uid, $rosternode) {
  global $user;
  $sql = "UPDATE {ult_roster_invites} set response=1, rejectdate=" . time() . " WHERE uid=%d AND roster=%d AND referrer=%d";
  $result = db_query($sql, $uid, $rosternode->nid, $uid);
  drupal_set_message(t("Roster application for !player declined.", array('!player' => ult_playername_theme($uid, TRUE))));
  // Determine whether 'reject app' notifications are required
  $targets = array();
  $targets = variable_get('ult_withdrawappnotify', array('player' => 'player'));
  if (($targets['player'] === 'player') || ($targets['captain'] === 'captain') || ($targets['team'] === 'team')) {
    drupal_set_message(t('Email notifications sent.'), 'status');
    $params = array();
    $params['teamnid'] = $rosternode->field_ult_rosterteam[0]['nid'];
    $teamnode = node_load($rosternode->field_ult_rosterteam[0]['nid']);
    $params['teamname'] = check_plain(node_page_title($teamnode));
    $params['name'] = ult_playername_theme($uid, FALSE);
    $params['userid'] = $uid;
    _ult_rosters_sendnotifications($targets, $params, 'rejectapp', $rosternode, $teamnode);
  }
}


/**
 * Allows a player to withdraw their application to join a roster
 *
 * @param $uid
 *   The uid of the user invited to join a particular roster
 * @param $rosterid
 *   The nid of the roster the invite is for
 */
function _ult_rosters_withdrawapp($uid, $rosternode) {
  global $user;
  $sql = "DELETE from {ult_roster_invites} WHERE uid=%d AND roster=%d AND referrer=%d";
  $result = db_query($sql, $uid, $rosternode->nid, $uid);
  drupal_set_message(t("Roster application for !player withdrawn.", array('!player' => ult_playername_theme($uid, TRUE))));
  // Determine whether 'new player joined' notifications are required
  $targets = array();
  if (variable_get('ult_enablecaptains', 1) == 1) {
    $targets = variable_get('ult_withdrawappnotify', array('captain' => 'captain'));
  }
  else {
    $targets = variable_get('ult_withdrawappnotify', array());
  }
  if (($targets['player'] === 'player') || ($targets['captain'] === 'captain') || ($targets['team'] === 'team')) {
    drupal_set_message(t('Email notifications sent.'), 'status');
    $params = array();
    $params['teamnid'] = $rosternode->field_ult_rosterteam[0]['nid'];
    $params['teamname'] = check_plain(node_page_title(node_load($params['teamnid'])));
    $params['name'] = ult_playername_theme($uid, FALSE);
    $params['userid'] = $uid;
    $teamnode = node_load($rosternode->field_ult_rosterteam[0]['nid']);
    _ult_rosters_sendnotifications($targets, $params, 'withdrawapp', $rosternode, $teamnode);
  }
}

/**
 * Deletes a particular player application
 *
 * @param $uid
 *   The uid of the user invited to join a particular roster
 * @param $rosterid
 *   The nid of the roster the invite is for
 */
function _ult_rosters_deleteapp($uid, $rosternode) {
  $sql = "DELETE from {ult_roster_invites} WHERE uid=%d AND roster=%d AND referrer=%d";
  $result = db_query($sql, $uid, $rosternode->nid, $uid);
}

/**
 * Withdraws a particular email roster invitation
 *
 * @param $rosternid
 *   The nid of the roster the invite is for
 * @param $name
 *   The name of the player invited to join a particular roster
 */
function _ult_rosters_withdrawemailinvite($rosternid, $playername) {
  $playername = urldecode($playername);
  $sql = "DELETE from {ult_roster_unknown_invites} where concat(firstname, lastname) like '%s' and roster = %d";
  $result = db_query($sql, $playername, $rosternid);
  drupal_set_message(t('Roster invitation withdrawn.'), 'status');
}
